import os
import sys
import Tigger
import numpy
import tempfile
import json
import codecs
import shlex
import shutil
import glob
import subprocess

from astLib.astWCS import WCS
from Tigger.Models import SkyModel, ModelClasses


CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]
OUTPUT = os.environ["OUTPUT"]

with codecs.open(CONFIG, "r", "utf8") as stdr:
    cab = json.load(stdr)

junk = cab["junk"]
args = []
msname = None

sofia_file = 'sofia_parameters.par'
wstd = open(sofia_file, 'w')

wstd.write('writeCat.outputDir={:s}\n'.format(OUTPUT))
port2tigger = False
image = None
writecat = False
parameterise = False

for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if value is None:
        continue
    if name == "port2tigger":
        port2tigger = value
        continue
    if name == "steps.doWriteCat":
        writecat = value
    if name == "steps.doParameterise":
        parameterise = value
    if name == "import.inFile":
        image = value

    wstd.write('{0}={1}\n'.format(name, value))

wstd.close()

_runc = " ".join(['sofia_pipeline.py', sofia_file])

try:
    subprocess.check_call(shlex.split(_runc))
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)
                # Leave other types

if not port2tigger:
    sys.exit(0)

# convert to data file to Tigger LSM
# First make dummy tigger model
tfile = tempfile.NamedTemporaryFile(suffix='.txt')
tfile.flush()

if image and writecat and parameterise:
    pass
else:
    sys.exit(0)

prefix = os.path.splitext(image)[0]
tname_lsm = prefix + ".lsm.html"
with open(tfile.name, "w") as stdw:
    stdw.write("#format:name ra_d dec_d i emaj_s emin_s pa_d\n")

model = Tigger.load(tfile.name)
tfile.close()


def tigger_src(src, idx):

    name = "SRC%d" % idx
    flux = ModelClasses.Polarization(src["f_int"], 0, 0, 0)
    ra = numpy.deg2rad(src["ra"])
    dec = numpy.deg2rad(src["dec"])
    pos = ModelClasses.Position(ra, dec)
    ex = numpy.deg2rad(src["ell_maj"])
    ey = numpy.deg2rad(src["ell_min"])
    pa = numpy.deg2rad(src["ell_pa"])

    if ex and ey:
        shape = ModelClasses.Gaussian(ex, ey, pa)
    else:
        shape = None
    source = SkyModel.Source(name, pos, flux, shape=shape)
    # Adding source peak flux (error) as extra flux attributes for sources,
    # and to avoid null values for point sources I_peak = src["Total_flux"]
    if shape:
        source.setAttribute("I_peak", float(src["f_peak"]))
    else:
        source.setAttribute("I_peak", float(src["f_int"]))

    return source


with open('{0}_cat.ascii'.format(prefix)) as stdr:
    # Header
    stdr.readline()
    # Column names
    names = stdr.readline().split("#")[1].strip().split()
    # Units
    stdr.readline()
    # Column numbers
    stdr.readline()
    sys.stdout.write(" ".join(names))
    data = numpy.genfromtxt(stdr,
                            names=names + ["col"])

for i, src in enumerate(data):
    model.sources.append(tigger_src(src, i))

wcs = WCS(image)
centre = wcs.getCentreWCSCoords()
model.ra0, model.dec0 = map(numpy.deg2rad, centre)

model.save(tname_lsm)
# Rename using CORPAT
utils.xrun("tigger-convert", [tname_lsm, "--rename -f"])



import os
import sys
import yaml
import subprocess
import shlex
import glob
import shutil

CONFIG = os.environ["CONFIG"]
OUTPUT = os.environ["OUTPUT"]
INDIR = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

params = cab["parameters"]
junk = cab["junk"]

_positional = ['antenna-file']
positional = []
for item in _positional:
    param = filter(lambda a: a['name'] == item, params)[0]
    value = param['value']
    params.remove(param)

    if value:
        positional.append(value)

args = []
for param in params:
    key = param['name']
    value = param['value']
    if value in [None, False]:
        continue
    if value is True:
        arg = '{0}{1}'.format(cab['prefix'], key)
    elif hasattr(value, '__iter__'):
        arg = ' '.join(['{0}{1} {2}'.format(cab['prefix'], key, val)
                        for val in value])
    else:
        if key in ["pol", "feed"]:
            value = '"{}"'.format(value)
        arg = '{0}{1} {2}'.format(cab['prefix'], key, value)
    args.append(arg)

_runc = " ".join([cab['binary']] + args + ['--nolog'] + positional)

try:
    subprocess.check_call(shlex.split(_runc))
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)
                # Leave other types



import os
import sys
import yaml
import shlex
import shutil
import subprocess
import glob


CONFIG = os.environ["CONFIG"]
OUTPUT = os.environ["OUTPUT"]
INPUT = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

junk = cab["junk"]

args = []
msname = None

custom_script = "print(\"Nothing has been done\")"

for param in cab['parameters']:
    name = param['name']
    value = param['value']
    if name == "script":
        custom_script = value
        continue
    if value is None:
        continue
    elif value is False:
        continue
    elif value is True:
        value = ''
    args += ['{0}{1} {2}'.format(cab['prefix'], name, value)]

with open("casajob.py.last", "w") as f:
    f.write(custom_script)


_runc = " ".join([cab['binary']] + ["-c", "casajob.py.last"] + args)
try:
    subprocess.check_call(shlex.split(_runc))
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)



import os
import sys
import yaml
import sharpener
import glob
import shlex
import subprocess


CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]
OUTPUT = os.environ["OUTPUT"]

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

junk = cab["junk"]
args = []
msname = None

pkg_path = os.path.dirname(os.path.realpath(sharpener.__file__))
sharpener_file = '{:s}/sharpener_default.yml'.format(pkg_path)

with open(sharpener_file) as f:
    list_doc = yaml.load(f)

for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if value is None:
        continue

    for key, val in list_doc.items():
        if type(val) == dict:
            for k1, v1 in val.items():
                if 'enable' in name:
                    if key in name:
                        list_doc[key]['enable'] = value
                elif k1 == name:
                    list_doc[key][k1] = value
        else:
            if key == name:
                list_doc[key] = value

# Get the relative path from workdir
list_doc['general']['contname'] = os.path.relpath(
        list_doc['general']['contname'], list_doc['general']['workdir'])
list_doc['general']['cubename'] = os.path.relpath(
        list_doc['general']['cubename'], list_doc['general']['workdir'])
list_doc['source_catalog']['catalog_file'] = os.path.relpath(
        list_doc['source_catalog']['catalog_file'], list_doc['general']['workdir'])

edited_file = 'sharpener_default.yml'
with open(edited_file, "w") as f:
    yaml.dump(list_doc, f)

_runc = "run_sharpener -c %s" % edited_file
try:
    subprocess.check_call(shelx.split(_runc))
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)
                # Leave other types



import sys
import os
from MSUtils import flag_stats
import inspect
import glob
import shutil
import yaml
import codecs
import json

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]
MSDIR = os.environ["MSDIR"]

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

junk = cab["junk"]
args = {}
for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if name in ["fields", "antennas"] and value is not None:
        try:
            value = list(map(int, value))
        except ValueError:
            pass
    if name == "outfile":
        outfile = value
        continue
    args[name] = value

try:
    stats = flag_stats.antenna_flags_field(**args)
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)

with codecs.open(outfile, 'w', 'utf8') as stdw:
    a = json.dumps(stats, ensure_ascii=False)
    stdw.write(a)



import os
import sys
import shlex
import shutil
import subprocess
import glob
import yaml

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
MSDIR = os.environ["MSDIR"]
OUTPUT = os.environ["OUTPUT"]

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

junk = cab["junk"]

args = {}
parset = []

for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if value is None:
        continue
    elif value is False:
        value = 0
    elif value is True:
        value = 1
    elif name == 'parset':
        parset = [value]
        continue
    elif isinstance(value, list):
        value = ",".join(map(str, value))

    args[name] = value

# available jones terms
joneses = "g b dd".split()
soljones = args.pop("sol-jones")

for jones in joneses:
    if jones.lower() not in soljones.lower():
        jopts = filter(lambda a: a.startswith(
            "{0:s}-".format(jones)), args.keys())
        for item in list(jopts):
            del args[item]

opts = ["{0:s}sol-jones {1:s}".format(cab["prefix"], soljones)] + \
    ['{0}{1} {2}'.format(cab['prefix'], name, value)
     for name, value in args.items()]

_runc = " ".join([cab["binary"]] + parset + opts)
try:
    subprocess.check_call(shlex.split(_runc))
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)



import sys
import os
import astropy.io.fits as pyfits
import glob
import subprocess
import shutil
import shlex
import yaml


CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]
MSDIR = os.environ["MSDIR"]

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

junk = cab["junk"]

args = []
parset = None
for param in cab['parameters']:
    name = param['name']
    value = param['value']
    if name == 'Parset' and value is not None:
        parset = value
        continue
    if name == 'Parset' and value is None:
        continue
    if name == 'Noise-Image' and value is None:
        continue

    if isinstance(value, list):
        arg = "{0}{1} {2}".format(cab['prefix'], name, ",".join(value))
    else:
        arg = '{0}{1} {2}'.format(cab['prefix'], name, value)

    args.append(arg)


removed = False
for item1 in args:
    if 'Noise-Image' in item1:
        noise_image = item1.split('{0}Noise-Image '.format(cab['prefix']))[-1]
        args.remove('{0}Noise-Image {1}'.format(cab['prefix'], noise_image))
        noise_hdu = pyfits.open(noise_image)
        noise_data = noise_hdu[0].data
        noise_std = noise_data.std()
        noise_hdu.close()
        for item2 in args:
            if 'Noise-Sigma' in item2:
                noise_sigma = item2.split(
                    '{0}Noise-Sigma '.format(cab['prefix']))[-1]
                args.remove(
                    '{0}Noise-Sigma {1}'.format(cab['prefix'], noise_sigma))
                removed = True
                threshold = float(noise_sigma)*noise_std
                for item3 in args:
                    if '{0}Deconv-FluxThreshold'.format(cab['prefix']) in item3:
                        args.remove(item3)
                args.append(
                    '{0}Deconv-FluxThreshold {1}'.format(cab['prefix'], threshold))
if not removed:
    args.remove('{0}Noise-Sigma 3.0'.format(cab['prefix']))

if parset is not None:
    args.insert(0, parset)


_runc = " ".join([cab['binary']] + args)

try:
    subprocess.check_call(shlex.split(_runc))
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)
                # Leave other types



import os
import sys
import logging
import Crasa.Crasa as crasa
import astropy.io.fits as pyfits
import glob
import yaml
import shutil


CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]
MSDIR = os.environ["MSDIR"]

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

junk = cab["junk"]

args = {}
for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if value is None:
        continue

    args[name] = value

noise_image = args.pop('noise_image', False)
if noise_image:
    noise_sigma = args.pop('noise_sigma')
    noise_hdu = pyfits.open(noise_image)
    noise_data = noise_hdu[0].data
    noise_std = noise_data.std()
    threshold = noise_sigma*noise_std
    args['threshold'] = '{}Jy'.format(threshold)
else:
    args.pop('noise_sigma')


prefix = args['imagename']
port2fits = args.pop('port2fits', True)
keep_casa_images = args.pop("keep_casa_images", False)

task = crasa.CasaTask(cab["binary"], **args)
try:
    task.run()
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)
                # Leave other types

nterms = args.get("nterms", 1)
images = ["flux", "model", "residual", "psf", "image"]
STD_IMAGES = images[:4]

convert = []
if port2fits:
    for image in images:
        img = "{:s}.{:s}".format(prefix, image)
        if image == 'flux':
            _images = [img]
        elif nterms > 1:
            _images = ["%s.tt%d" % (img, d) for d in range(nterms)]
            if image == "image":
                if nterms == 2:
                    alpha = img+".alpha"
                    alpha_err = img+".alpha.error"
                    _images += [alpha, alpha_err]
                if nterms == 3:
                    beta = img+".beta"
                    beta_err = img+".beta.error"
                    _images += [beta, beta_err]
        else:
            _images = [img]
        convert += _images

for _image in convert:
    sys.stdout.write(_image)
    if _image in STD_IMAGES and (not os.path.exists(_image)):
        raise RuntimeError(
            "Standard output from CASA clean task not found. Something went wrong durring cleaning, take a look at the logs and such")

    elif os.path.exists(_image):
        task = crasa.CasaTask(
            "exportfits", **dict(imagename=_image, fitsimage=_image+".fits", overwrite=True))
        try:
            task.run()
        finally:
            for item in junk:
                for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
                    items = glob.glob("{dest}/{item}".format(**locals()))
                    for f in items:
                        if os.path.isfile(f):
                            os.remove(f)
                        elif os.path.isdir(f):
                            shutil.rmtree(f)
                        # Leave other types

if not keep_casa_images:
    for _image in convert:
        os.system("rm -rf {}".format(_image))



import pyrap.images
import os
import sys
from pyrap.tables import table
from MSUtils import msutils
import tempfile
import astropy.io.fits as pyfits
import subprocess
import shlex
import shutil
import yaml


CONFIG = os.environ['CONFIG']
OUTPUT = os.environ['OUTPUT']
INPPUT = os.environ['INPUT']
MSDIR = os.environ['MSDIR']

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

junk = cab["junk"]

params = cab['parameters']

tdir = tempfile.mkdtemp(dir='.')
os.chdir(tdir)


def rm_fr(item):
    os.system('rm -fr {}'.format(item))


def _run(prefix=None, predict=False, **kw):

    port2fits = kw.pop('port2fits', True)
    keep_casa_images = kw.pop('keep_casa_images', False)

    if predict:
        args = ['{0}={1}'.format(a, b) for a, b in kw.iteritems()]
        _runc = " ".join([cab["binary"]] + args)
        subprocess.check_call(shlex.split(_runc))
        return

    if kw.get('niter', 0) > 0:
        if kw.get('operation', None) not in ['clark', 'hogbom', 'csclean', 'multiscale', 'entropy']:
            kw['operation'] = 'csclean'
        images = {
            "restored":   ['{0}.restored.{1}'.format(prefix, a) for a in ['fits', 'img']],
            "model":   ['{0}.model.{1}'.format(prefix, a) for a in ['fits', 'img']],
            "residual":   ['{0}.residual.{1}'.format(prefix, a) for a in ['fits', 'img']],
        }

    elif kw.get('niter', 0) == 0:
        kw["operation"] = 'image'

        images = {
            "image":   ['{0}.dirty.{1}'.format(prefix, a) for a in ['fits', 'img']],
        }

    for key, value in images.iteritems():
        kw[key] = value[1]

    args = ['{0}={1}'.format(a, b) for a, b in kw.iteritems()]
    _runc = " ".join([cab["binary"]] + args)
    subprocess.check_call(shlex.split(_runc))

    if port2fits:
        print('Converting CASA iamges to FITS images')
        for fits, img in images.itervalues():
            im = pyrap.images.image(img)
            im.tofits(fits, overwrite=True, velocity=kw.get(
                'prefervelocity', False))
            if not keep_casa_images:
                rm_fr(img)


def predict_vis(msname, image, column="MODEL_DATA",
                chanchunk=None, chanstart=0, chanstep=1):
    """Converts image into predicted visibilities"""

    # CASA to convert them
    casaimage = '{0}/{1}.img'.format(OUTPUT, os.path.basename(image))

    # convert to CASA image
    img = pyrap.images.image(image)
    img.saveas(casaimage)

    imgshp = img.shape()
    ftab = table(msname+'/SPECTRAL_WINDOW')
    numchans = ftab.getcol('NUM_CHAN')[0]
    # default chunk list is entire chanel range. Update this if needed
    chunklist = [(0, numchans, None, None)]
    if len(imgshp) == 4 and imgshp[0] > 1:
        nimgchan = imgshp[0]

        print("image cube has {0} channels, MS has {1} channels".format(
            nimgchan, numchans))

        imgchansize = imgshp[1]*imgshp[2]*imgshp[3] * \
            4  # size of an image channel in bytes
        if chanchunk is None:
            mem_bytes = os.sysconf('SC_PAGE_SIZE') * \
                os.sysconf('SC_PHYS_PAGES')  # e.g. 4015976448
            chanchunk = max((mem_bytes/20)/imgchansize, 1)
            print("based on available memory ({0}), max image chunk is {1} channels".format(
                mem_bytes, chanchunk))
        if chanchunk < nimgchan:
            mschanstep = numchans*chanstep/nimgchan
            if numchans % nimgchan:
                warn(
                    "MS channels not evenly divisible into $nimgchan image channels, chunking may be incorrect")
            chunklist = []
            for chan0 in range(0, nimgchan, chanchunk):
                imch0, imch1 = chan0, (min(chan0+chanchunk, nimgchan)-1)
                msch0 = chanstart + imch0*mschanstep
                msnch = (imch1-imch0+1)*mschanstep/chanstep
                # overlap each chunk from 1 onwards by a half-chunk back to take care of extrapolated visibilties
                # from previous channel
                if imch0:
                    imch0 -= 1
                    msch0 -= mschanstep/2
                    msnch += mschanstep/2
                print("image chunk {0}~{1} corresponds to MS chunk {2}~{3}".format(
                    imch0, imch1, msch0, msch0+msnch-1))
                chunklist.append((msch0, msnch, imch0, imch1))

    # even in fill-model mode where it claims to ignore image parameters, the image channelization
    # arguments need to be "just so" as per below, otherwise it gives a GridFT: weights all zero message
    kw0 = {}
    kw0.update(ms=msname, model=casaimage,
               niter=0, fixed=1, mode="channel", operation="csclean",
               img_nchan=1, img_chanstart=chanstart, img_chanstep=numchans*chanstep)
    kw0['fillmodel'] = 1

    blc = [0]*len(imgshp)
    trc = [x-1 for x in imgshp]

    # now loop over image frequency chunks
    for ichunk, (mschanstart, msnumchans, imgch0, imgch1) in enumerate(chunklist):
        if len(chunklist) > 1:
            blc[0], trc[0] = imgch0, imgch1
            print("writing CASA image for slice {0} {1}".format(blc, trc))
            casaimage1 = II("{0}.{1}.img".format(image, ichunk1))
            rm_fr(casaimage1)
            print("writing CASA image for slice {0} {1} to {2}".format(
                blc, trc, casaimage1))
            img.subimage(blc, trc, dropdegenerate=False).saveas(casaimage1)
            kw0.update(model=casaimage1)
        else:
            img.unlock()
        # setup imager options
        kw0.update(chanstart=mschanstart, chanstep=chanstep, nchan=msnumchans)
        print("predicting visibilities into MODEL_DATA")

        _run(predict=True, **kw0)
        if len(chunklist) > 1:
            rm_fr(casaimage1)
    rm_fr(casaimage)

    if column != "MODEL_DATA":
        print('Data was predicted to MODEL_DATA column. Will now copy it to the {} column as requested'.format(column))
        msutils.copycol(msname=msname, fromcol="MODEL_DATA", tocol=column)


options = {}
for param in params:
    value = param['value']
    name = param['name']

    if name == 'prefix':
        prefix = value
        continue

    if value is None:
        continue
    if name == 'cellsize':
        if isinstance(value, (float, int)):
            value = '{}arcsec'.format(value)
    elif name in ['threshold', 'targetflux']:
        if isinstance(value, float):
            value = '{}arcsec'.format(value)
    options[name] = value

noise_image = options.pop('noise_image', False)
if noise_image:
    noise_sigma = options.pop('noise_sigma')
    noise_hdu = pyfits.open(noise_image)
    noise_data = noise_hdu[0].data
    noise_std = noise_data.std()
    threshold = noise_sigma*noise_std
    options['threshold'] = '{}Jy'.format(threshold)
else:
    options.pop('noise_sigma')

predict = options.pop('simulate_fits', False)
if predict:
    tfile = tempfile.NamedTemporaryFile(suffix='.fits')
    tfile.flush()
    cell = options.get('cellsize', None)
    if cell is None:
        with pyfits.open(predict) as _hdu:
            if hasattr(_hdu, '__iter__'):
                hdu = _hdu[0]
            else:
                hdu = _hdu

            cdelt = hdu.header.get('CDELT1', None)
            if cdelt:
                cell = '{:f}arcsec'.format(abs(cdelt)*3600)

    if cell is None:
        raise RuntimeError('The size of a pixel in this FITS image was not specified \
in FITS header (CDELT1/2), or as parameter for this module ("cellsize"). Cannot proceed')

    _runc = " ".join(['python /scratch/code/predict_from_fits.py'] + [predict, options['ms'], cell,
                                                             tfile.name])
    subprocess.check_call(shlex.split(_runc))
    predict_vis(msname=options['ms'], image=tfile.name, column=options.get('data', 'MODEL_DATA'),
                chanchunk=options.get('chanchunk', None), chanstart=options.get('img_chanstart', 0),
                chanstep=options.get('img_chanstep', 1))
    tfile.close()
else:
    _run(prefix, **options)

os.chdir(OUTPUT)
os.system('rm -r {}'.format(tdir))



import os
import sys
import re
import bdsf as bdsm  # bdsm it is and bdsm it shall remain
import numpy
import Tigger
import tempfile
import astropy.io.fits as pyfits
import yaml
import shlex
import shutil
import glob
import subprocess

from astLib.astWCS import WCS
from Tigger.Models import SkyModel, ModelClasses

CONFIG = os.environ['CONFIG']
INPUT = os.environ['INPUT']
OUTPUT = os.environ['OUTPUT']
MSDIR = os.environ['MSDIR']

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)

junk = cab["junk"]

write_catalog = ['bbs_patches', 'bbs_patches_mask',
                 'catalog_type', 'clobber', 'correct_proj', 'format',
                 'incl_chan', 'incl_empty', 'srcroot', 'port2tigger', 'outfile']

img_opts = {}
write_opts = {}
# Spectral fitting parameters
freq0 = None
spi_do = False

for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if value is None:
        continue
    if name in ['multi_chan_beam']:
        multi_chan_beam = value
        continue
    if name in write_catalog:
        write_opts[name] = value
    elif name in ['freq0', 'frequency']:
        freq0 = value
    else:
        img_opts[name] = value
        if name == 'spectralindex_do':
            spi_do = value

img_opts.pop('freq0', None)
if freq0 is None:
    with pyfits.open(img_opts['filename']) as hdu:
        hdr = hdu[0].header
        for i in xrange(1, hdr['NAXIS']+1):
            if hdr['CTYPE{0:d}'.format(i)].startswith('FREQ'):
                freq0 = hdr['CRVAL{0:d}'.format(i)]

if spi_do and multi_chan_beam:
    with pyfits.open(img_opts['filename']) as hdu:
        hdr = hdu[0].header
    beams = []
    # Get a sequence of BMAJ with digit suffix from the image header keys
    bmaj_ind = filter(lambda a: a.startswith('BMAJ')
                      and a[-1].isdigit(), hdr.keys())
    for bmaj in bmaj_ind:
        ind = bmaj.split('BMAJ')[-1]
        beam = [hdr['{0:s}{1:s}'.format(b, ind)]
                for b in 'BMAJ BMIN BPA'.split()]
        beams.append(tuple(beam))
    # parse beam info to pybdsm
    img_opts['beam_spectrum'] = beams

image = img_opts.pop('filename')
filename = os.path.basename(image)

outfile = write_opts.pop('outfile')

try:
    img = bdsm.process_image(image, **img_opts)

    port2tigger = write_opts.pop('port2tigger', True)

    if port2tigger:
        write_opts['format'] = 'fits'

    img.write_catalog(outfile=outfile, **write_opts)
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)

if not port2tigger:
    sys.exit(0)

# convert to Gaul file to Tigger LSM
# First make dummy tigger model
tfile = tempfile.NamedTemporaryFile(suffix='.txt')
tfile.flush()

prefix = os.path.splitext(outfile)[0]
tname_lsm = prefix + ".lsm.html"
with open(tfile.name, "w") as stdw:
    stdw.write("#format:name ra_d dec_d i emaj_s emin_s pa_d\n")

model = Tigger.load(tfile.name)
tfile.close()


def tigger_src(src, idx):

    name = "SRC%d" % idx

    flux = ModelClasses.Polarization(
        src["Total_flux"], 0, 0, 0, I_err=src["E_Total_flux"])
    ra, ra_err = map(numpy.deg2rad, (src["RA"], src["E_RA"]))
    dec, dec_err = map(numpy.deg2rad, (src["DEC"], src["E_DEC"]))
    pos = ModelClasses.Position(ra, dec, ra_err=ra_err, dec_err=dec_err)
    ex, ex_err = map(numpy.deg2rad, (src["DC_Maj"], src["E_DC_Maj"]))
    ey, ey_err = map(numpy.deg2rad, (src["DC_Min"], src["E_DC_Min"]))
    pa, pa_err = map(numpy.deg2rad, (src["PA"], src["E_PA"]))

    if ex and ey:
        shape = ModelClasses.Gaussian(
            ex, ey, pa, ex_err=ex_err, ey_err=ey_err, pa_err=pa_err)
    else:
        shape = None
    source = SkyModel.Source(name, pos, flux, shape=shape)
    # Adding source peak flux (error) as extra flux attributes for sources,
    # and to avoid null values for point sources I_peak = src["Total_flux"]
    if shape:
        source.setAttribute("I_peak", src["Peak_flux"])
        source.setAttribute("I_peak_err", src["E_peak_flux"])
    else:
        source.setAttribute("I_peak", src["Total_flux"])
        source.setAttribute("I_peak_err", src["E_Total_flux"])

    if spi_do:
        # Check if start frequency is provided if not provided raise error.
        # It is used to define tigger source spectrum index frequency
        if freq0:
            spi, spi_err = (src['Spec_Indx'], src['E_Spec_Indx'])
            source.spectrum = ModelClasses.SpectralIndex(spi, freq0)
            source.setAttribute('spi_error', spi_err)
        else:
            raise RuntimeError("No start frequency (freq0) provided.")
    return source


with pyfits.open(outfile) as hdu:
    data = hdu[1].data

    for i, src in enumerate(data):
        model.sources.append(tigger_src(src, i))

wcs = WCS(image)
centre = wcs.getCentreWCSCoords()
model.ra0, model.dec0 = map(numpy.deg2rad, centre)

model.save(tname_lsm)
# Rename using CORPAT
_runc = "tigger-convert %s --rename -f" % tname_lsm
try:
    subprocess.check_call(shlex.split(_runc))
finally:
    for item in junk:
        for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
            items = glob.glob("{dest}/{item}".format(**locals()))
            for f in items:
                if os.path.isfile(f):
                    os.remove(f)
                elif os.path.isdir(f):
                    shutil.rmtree(f)



# -*- coding: future_fstrings -*-
import os
import sys
import inspect
import pkg_resources
import logging
from logging import StreamHandler
import re
from pathlib import Path

try:
    __version__ = pkg_resources.require("stimela")[0].version
except pkg_resources.DistributionNotFound:
    __version__ = "dev"

# Get to know user
USER = os.environ["USER"]
UID = os.getuid()
GID = os.getgid()
CAB_USERNAME = re.sub('[^0-9a-zA-Z]+', '_', USER).lower() 

root = os.path.dirname(__file__)

CAB_PATH = os.path.join(root, "cargo/cab")
BASE_PATH = os.path.join(root, "cargo/base")

# Set up logging infrastructure
LOG_HOME = os.path.expanduser("~/.stimela")
# make sure directory exists
Path(LOG_HOME).mkdir(exist_ok=True)
# This is is the default log file. It logs stimela images, containers and processes
LOG_FILE = "{0:s}/stimela_logfile.json".format(LOG_HOME)

GLOBALS = {'foo': 'bar'}
del GLOBALS['foo']

def register_globals():
    frame = inspect.currentframe().f_back
    frame.f_globals.update(GLOBALS)

# Get base images
# All base images must be on dockerhub
BASE = os.listdir(BASE_PATH)
CAB = list()

for item in os.listdir(CAB_PATH):
    try:
        # These files must exist for a cab image to be valid
        ls_cabdir = os.listdir('{0}/{1}'.format(CAB_PATH, item))
        paramfile = 'parameters.json' in ls_cabdir
        srcdir = 'src' in ls_cabdir
    except OSError:
        continue
    if paramfile and srcdir:
        CAB.append(item)


_logger = None

from .utils.logger import SelectiveFormatter, ColorizingFormatter, ConsoleColors, MultiplexingHandler

log_console_handler = log_formatter = log_boring_formatter = log_colourful_formatter = None

def logger(name="STIMELA", propagate=False, console=True, boring=False,
           fmt="{asctime} {name} {levelname}: {message}",
           col_fmt="{asctime} {name} %s{levelname}: {message}%s"%(ConsoleColors.BEGIN, ConsoleColors.END),
           sub_fmt="# {message}",
           col_sub_fmt="%s# {message}%s"%(ConsoleColors.BEGIN, ConsoleColors.END),
           datefmt="%Y-%m-%d %H:%M:%S", loglevel="INFO"):
    """Returns the global Stimela logger (initializing if not already done so, with the given values)"""
    global _logger
    if _logger is None:
        _logger = logging.getLogger(name)
        _logger.setLevel(getattr(logging, loglevel))
        _logger.propagate = propagate

        global log_console_handler, log_formatter, log_boring_formatter, log_colourful_formatter

        # this function checks if the log record corresponds to stdout/stderr output from a cab
        def _is_from_subprocess(rec):
            return hasattr(rec, 'stimela_subprocess_output')

        log_boring_formatter = SelectiveFormatter(
                    logging.Formatter(fmt, datefmt, style="{"),
                    [(_is_from_subprocess, logging.Formatter(sub_fmt, datefmt, style="{"))])

        log_colourful_formatter = SelectiveFormatter(
                    ColorizingFormatter(col_fmt, datefmt, style="{"),
                    [(_is_from_subprocess, ColorizingFormatter(fmt=col_sub_fmt, datefmt=datefmt, style="{",
                                                               default_color=ConsoleColors.DIM))])

        log_formatter = log_boring_formatter if boring else log_colourful_formatter

        if console:
            if "SILENT_STDERR" in os.environ and os.environ["SILENT_STDERR"].upper()=="ON":
                log_console_handler = StreamHandler(stream=sys.stdout)
            else:  
                log_console_handler = MultiplexingHandler()
            log_console_handler.setFormatter(log_formatter)
            log_console_handler.setLevel(getattr(logging, loglevel))
            _logger.addHandler(log_console_handler)

    return _logger

from stimela.recipe import Recipe




import os
import sys
import logging
import Crasa.Crasa as crasa
import yaml
import glob
import shutil

CONFIG = os.environ["CONFIG"]
INPUT = os.environ["INPUT"]
OUTPUT = os.environ["OUTPUT"]
MSDIR = os.environ["MSDIR"]

with open(CONFIG, "r") as _std:
    cab = yaml.safe_load(_std)
junk = cab["junk"]

unstack_params = ['unstack', 'nchans', 'keep_casa_images', 'port2fits']
unstack_args = {}
immath_args = {}


def rm_fr(item):
    os.system('rm -fr {}'.format(item))


for param in cab['parameters']:
    name = param['name']
    value = param['value']

    if value is None:
        continue

    if name in unstack_params:
        unstack_args[name] = value
    else:
        immath_args[name] = value

unstack = unstack_args.pop(cab["binary"], False)

if not unstack:
    task = crasa.CasaTask("immath", **immath_args)
    try:
        task.run()
    finally:
        for item in junk:
            for dest in [OUTPUT, MSDIR]: # these are the only writable volumes in the container
                items = glob.glob("{dest}/{item}".format(**locals()))
                for f in items:
                    if os.path.isfile(f):
                        os.remove(f)
                    elif os.path.isdir(f):
                        shutil.rmtree(f)
else:
    images = immath_args['imagename']
    for image in images:
        for i in range(unstack_args['nchans']):
            ext = image.split('.')[-1]
            chan_num = str(i)
            outfile = '{:s}-{:s}.{:s}'.format(
                immath_args['outfile'], chan_num, ext)
            run_immath_args = immath_args.copy()
            run_immath_args['imagename'] = image
            run_immath_args['outfile'] = outfile
            run_immath_args['chans'] = chan_num
            task = crasa.CasaTask(cab["binary"], **run_immath_args)
            task.run()
            if unstack_args['port2fits']:
                print('Converting CASA images to FITS images')
                fits = outfile + '.fits'
                task = crasa.CasaTask(
                    "exportfits", **dict(imagename=outfile, fitsimage=fits, overwrite=True))
                task.run()
                if not unstack_args['keep_casa_images']:
                    rm_fr(outfile)
